// ==============================================================
// ORBITER vessel module: Mercury Redstone
// ==============================================================

#include "orbitersdk.h"
#include "multi_stage.h"
#include "ariane5.h"


// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	Stack = new Multi_Stage();
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (Stack) delete Stack;
	if (vessel) delete vessel;
}

// Keyboard interface handler
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	Stack->ConsumeKey(vessel,keystate);
	return 0;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	IStage *newStage;
	newStage = new Ariane5BoosterStage();
	Stack->SetStage(newStage,vessel,0);
}


DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	Stack->Timestep(vessel,simt);
}

DLLCLBK void ovcLoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs)
{
	IStage *newStage;
    char *line;
	int status = 0;
	int jettisoned=0;
	double sts_sat_x = 0.0;
	double sts_sat_y = 0.0;
	double sts_sat_z = 0.0;
	bool configurationset = false;
	newStage = new Ariane5BoosterStage();
	Stack->SetStage(newStage,vessel,0);
	while (oapiReadScenario_nextline (scn, line)) {
		if (configurationset)
		{
			newStage->ParseLine(vessel,line,def_vs);
		}
		else
		{
			if (!strnicmp (line, "CONFIGURATION", 13)) {
				sscanf (line+13, "%d", &status);
				switch (status) {
				case 0:
					newStage = new Ariane5BoosterStage();
					break;
				case 1:
					newStage = new Ariane5FirstStage();
					break;
				}
				Stack->SetStage(newStage,vessel,0);
				configurationset = true;
			} else {
				vessel->ParseScenarioLine (line, def_vs);
			}
		}
    }
	newStage->Setup(vessel,0);
	Stack->SetCurrentStage(newStage);
}
//

// Save status to scenario file
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	vessel->SaveDefaultState(scn);
	Stack->SaveState(vessel,scn);
}
