#ifndef __ARIANE5_H
#define __ARIANE5_H
#include "orbitersdk.h"
#include "multi_stage.h"


const VECTOR3 OFS_LEFTPAYLOAD      = { 0,-2.69/2,9.09+15};
const VECTOR3 OFS_RIGHTPAYLOAD     = { 0, 2.69/2,9.09+15};
const VECTOR3 OFS_STAGE1       = { 0.0, 0.0, 0.0};
const VECTOR3 OFS_STAGE0_LEFT  = { -3.45, 0.0, OFS_STAGE1.z-2};
const VECTOR3 OFS_STAGE0_RIGHT = {  3.45, 0.0, OFS_STAGE1.z-2};

const VECTOR3 OFS_STAGE0_LEFT_FLAME = {OFS_STAGE0_LEFT.x-.2, 0.0, OFS_STAGE0_LEFT.z-14.0};
const VECTOR3 OFS_STAGE0_RIGHT_FLAME = {OFS_STAGE0_RIGHT.x+.2, 0.0,  OFS_STAGE0_RIGHT.z-14.0};
const VECTOR3 OFS_STAGE1_FLAME = { 0.0, 0.0,  OFS_STAGE1.z-15.0};

const double MASS_FIRST    = 170800*KG;
const double MASS_BOOSTER  = 277500*KG;
const double MASS_SHROUD   = 500*KG;

const double MASS_FIRST_EMPTY   = 12700*KG;
const double MASS_FIRST_FUEL    = MASS_FIRST-MASS_FIRST_EMPTY;
const double MASS_BOOSTER_EMPTY = 39800*KG;
const double MASS_BOOSTER_FUEL  = MASS_BOOSTER-MASS_BOOSTER_EMPTY;


const double ISP_BOOSTER_SL  = 250*SEC;
const double ISP_BOOSTER_VAC = 275*SEC;
const double ISP_FIRST_SL    = 340*SEC;
const double ISP_FIRST_VAC   = 430*SEC;

const double THRUST_BOOSTER_VAC = 660000*KGF;
const double THRUST_FIRST_VAC   = 113600*KGF;

VECTOR3 sat_ofs           = _V(0,0,0);
VECTOR3 ofs_sts_sat       = _V(0,0,0);
char sat_name[256]        ="";
char sat_mesh_name[256]   ="";
bool sat_jettisoned       =true;
double sat_mass	          = 0;

void AddLeftBoosterStage(VESSEL *vessel)
{
	vessel->AddMesh ("ariane5_leftsrb", &OFS_STAGE0_LEFT);
};


void AddRightBoosterStage(VESSEL *vessel)
{
	vessel->AddMesh ("ariane5_rightsrb", &OFS_STAGE0_RIGHT);
};

void AddLeftPayload(VESSEL *vessel)
{
	vessel->AddMesh ("ariane5_leftpay", &OFS_LEFTPAYLOAD);
};


void AddRightPayload(VESSEL *vessel)
{
	vessel->AddMesh ("ariane5_rightpay", &OFS_RIGHTPAYLOAD);
};

void AddFirstStage(VESSEL *vessel)
{
	vessel->AddMesh ("ariane5_main", &OFS_STAGE1);
};


class Ariane5FirstStage:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	void JettisonSatellite(VESSEL *vessel);
	void JettisonShroud(VESSEL *vessel);
	bool bManualSeparate;
	bool bAbort;
	bool doAbort;
	double slThrust;
	double vacThrust;
	bool ShroudJettisoned;
};

void Ariane5FirstStage::Init ()
{
	bManualSeparate = false;
	bAbort=false;
	doAbort=false;
	slThrust        = CalcSeaLevelThrust(THRUST_FIRST_VAC,ISP_FIRST_VAC,ISP_FIRST_SL,MASS_FIRST_FUEL);
	vacThrust       = THRUST_FIRST_VAC;
	ShroudJettisoned=false;
};
void Ariane5FirstStage::Exit (VESSEL *vessel)
{
};
void Ariane5FirstStage::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetStage(vessel);
};
void Ariane5FirstStage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	int jettisoned=0;
	double sts_sat_x = 0.0;
	double sts_sat_y = 0.0;
	double sts_sat_z = 0.0;
 	if (!strnicmp (line, "SAT_JETTISONED", 14)) {
        sscanf (line+14, "%d", &jettisoned);
		if (jettisoned==1) {
			sat_jettisoned=true;
		} else {
			sat_jettisoned=false;
		}
	} else if (!strnicmp (line, "SAT_OFS_X", 9)) {
		sscanf (line+9, "%lf", &sts_sat_x);
		ofs_sts_sat.x=sts_sat_x;
	} else if (!strnicmp (line, "SAT_OFS_Y", 9)) {
		sscanf (line+9, "%lf", &sts_sat_y);
		ofs_sts_sat.y=sts_sat_y;
	} else if (!strnicmp (line, "SAT_OFS_Z", 9)) {
		sscanf (line+9, "%lf", &sts_sat_z);
		ofs_sts_sat.z=sts_sat_z;
	} else if (!strnicmp (line, "SAT_NAME", 8)) {
		sscanf (line+8, "%s", sat_name);
	} else if (!strnicmp (line, "SAT_MESH", 8)) {
		sscanf (line+8, "%s", sat_mesh_name);
	} else if (!strnicmp (line, "SAT_MASS", 8)) {
		sscanf (line+8, "%lf", &sat_mass);
	} else if (!strnicmp (line, "SHROUD", 6)) {
		sscanf (line+6, "%d", &jettisoned);
		if (jettisoned==1) {
			ShroudJettisoned=true;
		} else {
			ShroudJettisoned=false;
		}
	} else {
		vessel->ParseScenarioLine (line, def_vs);
	}
};
void Ariane5FirstStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	int jettisoned = 0;
	oapiWriteScenario_int (scn, "CONFIGURATION", 1);
	oapiWriteScenario_float (scn, "SAT_OFS_X", ofs_sts_sat.x);
	oapiWriteScenario_float (scn, "SAT_OFS_Y", ofs_sts_sat.y);
	oapiWriteScenario_float (scn, "SAT_OFS_Z", ofs_sts_sat.z);
	oapiWriteScenario_float (scn, "SAT_MASS", sat_mass);
	if (sat_jettisoned)
		jettisoned=1;
	else
		jettisoned=0;
	oapiWriteScenario_int (scn, "SAT_JETTISONED", jettisoned);
	oapiWriteScenario_string (scn, "SAT_NAME", sat_name);
	oapiWriteScenario_string (scn, "SAT_MESH", sat_mesh_name);
	if (ShroudJettisoned)
		jettisoned=1;
	else
		jettisoned=0;
	oapiWriteScenario_int (scn, "SHROUD", jettisoned);
};
void Ariane5FirstStage::Timestep (VESSEL *vessel, double simt)
{
	OBJHANDLE hvessel;
	if (bAbort)
	{
		vessel->SetEngineLevel(ENGINE_MAIN,1.0);				
		bAbort = false;
		doAbort = true;
	}
	else if (doAbort)
	{
		doAbort=false;
		return;
	}
	else 
	{
		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			bManualSeparate=false;
			if (ShroudJettisoned)
			{
				if (!sat_jettisoned)
				{
					JettisonSatellite(vessel);
					sat_jettisoned=true;
					SetStage(vessel);
					vessel->SetEngineLevel(ENGINE_MAIN,0);
					char name[256];
					strcpy (name, vessel->GetName()); strcat (name, "-Sat");
					hvessel=oapiGetVesselByName(name);
					if (hvessel != 0)
						oapiSetFocusObject(hvessel);
				}
			}
			else
			{
				JettisonShroud(vessel);
				ShroudJettisoned=true;
				SetStage(vessel);
			}
			return;
		}
	}
	DoThrustAdjust(vessel,slThrust,vacThrust);
};
void Ariane5FirstStage::Animate (VESSEL *vessel, double simt)
{
};
void Ariane5FirstStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void Ariane5FirstStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int Ariane5FirstStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "Abort Emergency"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				bAbort = true;
			return 1;
		}
	}
	return 0;
};

void Ariane5FirstStage::SetStage(VESSEL *vessel)
{
	vessel->SetSize (15.0);
	vessel->SetCOG_elev (13);
	vessel->SetMaxFuelMass(MASS_FIRST_FUEL);
//	vessel->SetFuelMass(MASS_FIRST_FUEL);
	vessel->SetEmptyMass  (MASS_FIRST_EMPTY +MASS_SHROUD * 2);
	vessel->SetISP        (ISP_FIRST_VAC);
	vessel->SetMaxThrust  (ENGINE_MAIN, vacThrust);
	vessel->SetMaxThrust  (ENGINE_RETRO, 0);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 5e4);
//	vessel->SetPMI (_V(82,82.75+3.5,1.25+10.0+.85));
//	vessel->SetCrossSections (_V(70+40.25+12.5,70+62.75+12.5,19.75));
	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_STAGE1_FLAME.z), _V(-.7,.7,OFS_STAGE1_FLAME.z), _V(.7,.7,OFS_STAGE1_FLAME.z));
	vessel->SetCW (0.09, 0.09, 2.0, 1.4);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 mesh_dir;
	VECTOR3 mesh_pos;
	mesh_dir=_V(0,0,-1);
	mesh_pos=OFS_STAGE1_FLAME ;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,10,1,&mesh_dir);
	AddFirstStage(vessel);
	if (!ShroudJettisoned)
	{
		AddLeftPayload(vessel);
		AddRightPayload(vessel);
	}
	if (!sat_jettisoned)
	{
		vessel->AddMesh(sat_mesh_name,&ofs_sts_sat);
		vessel->SetEmptyMass(vessel->GetEmptyMass()+sat_mass);
	}
};

void Ariane5FirstStage::JettisonSatellite(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,-2.0);
	ofs = ofs_sts_sat;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Sat");
	VESSEL::Create (name, sat_name, vs);
};

void Ariane5FirstStage::JettisonShroud(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,-2.0,0);
	ofs = OFS_LEFTPAYLOAD;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_LeftPayload");
	VESSEL::Create (name, "Ariane5_LeftPayload", vs);
	vel = _V(0,2.0,0);
	ofs = OFS_RIGHTPAYLOAD;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_RightPayload");
	VESSEL::Create (name, "Ariane5_RightPayload", vs);
};

class Ariane5BoosterStage:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	void JettisonStage(VESSEL *vessel);
	void JettisonX20(VESSEL *vessel);
	void doBoosterTimeStep (VESSEL *vessel, double simt);
	bool bManualSeparate;
	bool bAbort;
	bool doAbort;
	double slThrust;
	double vacThrust;
	double slBooster;
	double vacBooster;
	double last_simt;
	double booster_fuel;
	double empty_mass;
	double booster_ignition_time;
	bool booster_ignited;
	double booster_sep_time;
	int boost_id1;
	int boost_id2;
};

void Ariane5BoosterStage::Init ()
{
	bManualSeparate = false;
	bAbort          = false;
	doAbort         = false;
	slThrust        = CalcSeaLevelThrust(THRUST_FIRST_VAC,ISP_FIRST_VAC,ISP_FIRST_SL,MASS_FIRST_FUEL);
	vacThrust       = THRUST_FIRST_VAC;
	slBooster       = CalcSeaLevelThrust(THRUST_BOOSTER_VAC,ISP_BOOSTER_VAC,ISP_BOOSTER_SL,MASS_BOOSTER_FUEL);
	vacBooster      = THRUST_BOOSTER_VAC;
	last_simt       = 0.0;
	booster_fuel    = MASS_BOOSTER_FUEL;
	empty_mass      = MASS_BOOSTER_EMPTY*2 + MASS_FIRST + MASS_SHROUD *2;
	booster_sep_time      = 129;
	booster_ignition_time = 0;
	booster_ignited       = false;
	Booster_Setup(booster_sep_time,MASS_BOOSTER_FUEL);
};

void Ariane5BoosterStage::Exit (VESSEL *vessel)
{
};

void Ariane5BoosterStage::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetStage(vessel);
};

void Ariane5BoosterStage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	int jettisoned=0;
	double old_ignition_time=0.0;
	double sts_sat_x = 0.0;
	double sts_sat_y = 0.0;
	double sts_sat_z = 0.0;
    if (!strnicmp (line, "ITIME", 5)) {
        sscanf (line+5, "%lf", &old_ignition_time);
		booster_ignition_time=-old_ignition_time;
 	} else if (!strnicmp (line, "BOOSTER_IGNITED", 15)) {
        sscanf (line+15, "%d", &jettisoned);
		if (jettisoned==1) {
			booster_ignited=true;
		} else {
			booster_ignited=false;
		}
 	} else if (!strnicmp (line, "SAT_JETTISONED", 14)) {
        sscanf (line+14, "%d", &jettisoned);
		if (jettisoned==1) {
			sat_jettisoned=true;
		} else {
			sat_jettisoned=false;
	}
	} else if (!strnicmp (line, "SAT_OFS_X", 9)) {
		sscanf (line+9, "%lf", &sts_sat_x);
		ofs_sts_sat.x=sts_sat_x;
	} else if (!strnicmp (line, "SAT_OFS_Y", 9)) {
		sscanf (line+9, "%lf", &sts_sat_y);
		ofs_sts_sat.y=sts_sat_y;
	} else if (!strnicmp (line, "SAT_OFS_Z", 9)) {
		sscanf (line+9, "%lf", &sts_sat_z);
		ofs_sts_sat.z=sts_sat_z;
	} else if (!strnicmp (line, "SAT_NAME", 8)) {
		sscanf (line+8, "%s", sat_name);
	} else if (!strnicmp (line, "SAT_MESH", 8)) {
		sscanf (line+8, "%s", sat_mesh_name);
	} else if (!strnicmp (line, "SAT_MASS", 8)) {
		sscanf (line+8, "%lf", &sat_mass);
    } else {
        vessel->ParseScenarioLine (line, def_vs);
    }
};

void Ariane5BoosterStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	int jettisoned = 0;
	oapiWriteScenario_int (scn, "CONFIGURATION", 0);
	oapiWriteScenario_float (scn, "ITIME", last_simt-booster_ignition_time);
	if (booster_ignited)
		jettisoned=1;
	else
		jettisoned=0;
	oapiWriteScenario_int (scn, "BOOSTER_IGNITED", jettisoned);
	oapiWriteScenario_float (scn, "SAT_OFS_X", ofs_sts_sat.x);
	oapiWriteScenario_float (scn, "SAT_OFS_Y", ofs_sts_sat.y);
	oapiWriteScenario_float (scn, "SAT_OFS_Z", ofs_sts_sat.z);
	if (sat_jettisoned)
		jettisoned=1;
	else
		jettisoned=0;
	oapiWriteScenario_int (scn, "SAT_JETTISONED", jettisoned);
	oapiWriteScenario_string (scn, "SAT_NAME", sat_name);
	oapiWriteScenario_string (scn, "SAT_MESH", sat_mesh_name);
	oapiWriteScenario_float (scn, "SAT_MASS", sat_mass);
};
void Ariane5BoosterStage::Timestep (VESSEL *vessel, double simt)
{
	double FuelLevel=0.0;
	double booster_time=0.0;
	IStage *newStage;
	
	if (bAbort)
	{
		vessel->SetEngineLevel(ENGINE_MAIN,1.0);				
		bAbort = false;
		doAbort = true;
	}
	else if (doAbort)
	{
		doAbort=false;
		return;
	}
	else 
	{
		if (!booster_ignited)
		{
			if (vessel->GetEngineLevel (ENGINE_MAIN) > 0.95) 
			{
				booster_ignited = true;
				booster_ignition_time = simt; 
			}
		}
		else
		{
			if (((simt-booster_ignition_time) > booster_sep_time) || bManualSeparate)
			{
				bManualSeparate=false;
				JettisonStage(vessel);
				newStage = new Ariane5FirstStage();
				Stack->SetStage(newStage,vessel,simt);
				vessel->SetEngineLevel(ENGINE_MAIN,1.0);
				return;
			};
		}
	}
	doBoosterTimeStep(vessel,simt);
	last_simt=simt;
};

void Ariane5BoosterStage::doBoosterTimeStep (VESSEL *vessel, double simt)
{
	double *booster_thrust;
	double c_booster_thrust;
	double thrust_level, prop_level;
	double booster_time=0.0;
	booster_thrust=vessel->GetMainThrustModPtr();	
	*booster_thrust = 0.0;
	if (booster_ignited)
	{
		booster_time = simt-booster_ignition_time;
		booster_thrust = vessel->GetMainThrustModPtr();
		GetBooster_State (booster_time, thrust_level, prop_level);
		c_booster_thrust = GetCurrentValue(vessel,slBooster,vacBooster)*2*thrust_level;
		vessel->SetEmptyMass(empty_mass+(booster_fuel*2*prop_level));
		//vessel->SetMaxThrust(ENGINE_RETRO,thrust_level);
		//vessel->SetMaxThrust(ENGINE_HOVER,booster_ignition_time);
		double exhaust_scale_1 = 16.0*thrust_level;
		double exhaust_scale_2 = 2.0*sqrt(thrust_level);
		vessel->SetExhaustScales (EXHAUST_CUSTOM, boost_id1, 10, 1);
		vessel->SetExhaustScales (EXHAUST_CUSTOM, boost_id2, 10, 1);
	}
	else
	{
		c_booster_thrust = 0.0;
	}
	*booster_thrust = (GetCurrentValue(vessel,slThrust,vacThrust)-vacThrust)+c_booster_thrust;
}

void Ariane5BoosterStage::Animate (VESSEL *vessel, double simt)
{
};
void Ariane5BoosterStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void Ariane5BoosterStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int Ariane5BoosterStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "Abort Emergency"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				bAbort = true;
			return 1;
		}
	}
	return 0;
};

void Ariane5BoosterStage::SetStage(VESSEL *vessel)
{
	vessel->SetSize (15.0);
	vessel->SetCOG_elev (13);
	vessel->SetMaxFuelMass(MASS_FIRST_FUEL);
	vessel->SetEmptyMass  (empty_mass + booster_fuel*2);
	vessel->SetISP        (ISP_FIRST_VAC);
	vessel->SetMaxThrust  (ENGINE_MAIN, vacThrust);
	vessel->SetMaxThrust  (ENGINE_RETRO, 0);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 5e4);

	//vessel->SetPMI (_V(40+18.5+3.5,40+24.75+3.5,1.25+10.0+.85));
	//vessel->SetCrossSections (_V(70+40.25+12.5,70+62.75+12.5,19.75));
	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_STAGE0_LEFT_FLAME.z), _V(-.7,.7,OFS_STAGE0_LEFT_FLAME.z), _V(.7,.7,OFS_STAGE0_LEFT_FLAME.z));
	vessel->SetCW (0.09, 0.09, 2.0, 1.4);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 mesh_dir;
	VECTOR3 mesh_pos;
	mesh_dir=_V(0,0,-1);
	mesh_pos=OFS_STAGE1_FLAME ;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,10,1,&mesh_dir);
	boost_id1 = vessel->AddExhaustRef (EXHAUST_CUSTOM,_V(OFS_STAGE0_LEFT_FLAME.x,OFS_STAGE0_LEFT_FLAME.y,OFS_STAGE0_LEFT_FLAME.z), 0, 0);
	boost_id2 = vessel->AddExhaustRef (EXHAUST_CUSTOM,_V(OFS_STAGE0_RIGHT_FLAME.x,OFS_STAGE0_RIGHT_FLAME.y,OFS_STAGE0_RIGHT_FLAME.z), 0, 0);
	AddLeftBoosterStage(vessel);
	AddRightBoosterStage(vessel);
	AddFirstStage(vessel);
	AddLeftPayload(vessel);
	AddRightPayload(vessel);
	if (!sat_jettisoned)
	{
		vessel->AddMesh (sat_mesh_name,&ofs_sts_sat);
		vessel->SetEmptyMass(vessel->GetEmptyMass()+sat_mass);
	}
};

void Ariane5BoosterStage::JettisonStage(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(-2.0,0,0);
	ofs = OFS_STAGE0_LEFT;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_LeftBooster");
	VESSEL::Create (name, "Ariane5_LeftBooster", vs);

	vel = _V(2.0,0,0);
	ofs = OFS_STAGE0_RIGHT;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_RightBooster");
	VESSEL::Create (name, "Ariane5_RightBooster", vs);
};


void Ariane5BoosterStage::JettisonX20(VESSEL *vessel)
{

};


#endif